@@ -285,98 +285,94 @@ module NumRu
    end
    
    # The put and get methods in the NetCDFVar class
+
+   def pack(na)
+     sf = att('scale_factor')
+     ao = att('add_offset')
+     if ( sf == nil && ao == nil ) then
+       na
+     else
+       na = NArray.to_na(na) if na.is_a?(Array)
+       if sf
+	 csf = sf.get
+	 raise NetcdfError,"scale_factor is not a numeric" if csf.is_a?(String)
+	 raise NetcdfError, "scale_factor is not unique" if csf.length != 1
+	 raise NetcdfError, "zero scale_factor" if csf[0] == 0
+       else
+	 csf = nil
+       end
+       if ao
+	 cao = ao.get
+	 raise NetcdfError, "add_offset is not a numeric" if cao.is_a?(String)
+	 raise NetcdfError, "add_offset is not unique" if cao.length != 1
+       else
+	 cao = nil
+       end
+       if csf and cao
+	 (na - cao) / csf
+       elsif csf
+	 na / csf
+       elsif cao
+	 na - cao
+       end
+     end
+   end
    
    def scaled_put(var,hash=nil)
-      sf = att('scale_factor')
-      ao = att('add_offset')
-      if ( sf == nil && ao == nil ) then
-	 # no scaling --> just call put
-	 simple_put(var,hash)
-      else
-	 if (sf != nil) 
-	    csf = sf.get
-	    if csf.is_a?(NArray) then  # --> should be a numeric
-	       csf = csf[0]
-	    elsif csf.is_a?(String)
-	       raise TypeError, "scale_factor is not a numeric"
-	    end
-	    if(csf == 0) then; raise NetcdfError, "zero scale_factor"; end
-	 else
-	    csf = 1.0      # assume 1 if not defined
-	 end
-	 if (ao != nil) 
-	    cao = ao.get
-	    if cao.is_a?(NArray) then  # --> should be a numeric
-	       cao = cao[0]
-	    elsif csf.is_a?(String)
-	       raise NetcdfError, "add_offset is not a numeric"
-	    end
-	 else
-	    cao = 0.0      # assume 0 if not defined
-	 end
-	 simple_put( (var-cao)/csf, hash )
-      end
+     simple_put( pack(var), hash)
    end
 
-   @@scaled_get_type = nil
+   @@unpack_type = nil
    class << NetCDFVar
-     def scaled_get_type
-       @@scaled_get_type
+     def unpack_type
+       @@unpack_type
      end
-     def scaled_get_type=(na_type)
+     def unpack_type=(na_type)
        if [NArray::BYTE, NArray::SINT, NArray::INT, 
-	   NArray::SFLOAT, NArray::FLOAT].include?(na_type)
-	 @@scaled_get_type = na_type
+	   NArray::SFLOAT, NArray::FLOAT, nil].include?(na_type)
+	 @@unpack_type = na_type
        else
 	 raise ArgumentError, "Arg must be one of NArray::BYTE, NArray::SINT, NArray::INT, NArray::SFLOAT, NArray::FLOAT"
        end
      end
+
+   end
+
+   def unpack(na)
+     sf = att('scale_factor')
+     ao = att('add_offset')
+     if ( sf == nil && ao == nil ) then
+       na
+     else
+       if sf
+	 csf = sf.get
+	 raise NetcdfError,"scale_factor is not a numeric" if csf.is_a?(String)
+	 raise NetcdfError, "scale_factor is not unique" if csf.length != 1
+	 raise NetcdfError, "zero scale_factor" if csf[0] == 0
+       else
+	 csf =nil
+       end
+       if ao
+	 cao = ao.get
+	 raise NetcdfError, "add_offset is not a numeric" if cao.is_a?(String)
+	 raise NetcdfError, "add_offset is not unique" if cao.length != 1
+       else
+	 cao = nil
+       end
+       if csf and cao
+	 una = na * csf + cao  # csf & cao are NArray -> coerced to their types
+       elsif csf
+	 una = na * csf
+       elsif cao
+	 una = na + cao
+       end
+       una = una.to_type(@@unpack_type) if @@unpack_type
+       una
+     end
    end
 
    def scaled_get(hash=nil)
-      sf = att('scale_factor')
-      ao = att('add_offset')
-      if ( sf == nil && ao == nil ) then
-	 # no scaling --> just call get
-	 simple_get(hash)
-      else
-	 if (sf != nil) 
-	    csf = sf.get
-	    if csf.is_a?(NArray) then  # --> should be a numeric
-	       csf = csf[0]
-	    elsif csf.is_a?(String)
-	       raise NetcdfError, "scale_factor is not a numeric"
-	    end
-	    if(csf == 0) then; raise NetcdfError, "zero scale_factor"; end
-	 else
-	    csf = 1.0      # assume 1 if not defined
-	 end
-	 if (ao != nil) 
-	    cao = ao.get
-	    if cao.is_a?(NArray) then  # --> should be a numeric
-	       cao = cao[0]
-	    elsif csf.is_a?(String)
-	       raise NetcdfError, "add_offset is not a numeric"
-	    end
-	 else
-	    cao = 0.0      # assume 0 if not defined
-	 end
-	 var = simple_get(hash)
-	 case (vartype)
-	 when "int","float","sfloat"
-	    tp = @@scaled_get_type || NArray::FLOAT
-	 when "byte","sint","char"
-	    tp = @@scaled_get_type || NArray::SFLOAT
-	 else
-	   raise "Sorry, must be a bag"
-	 end
-	 if tp <= NArray::INT && Float===csf
-	   ( var.to_type(NArray::FLOAT) * csf + cao ).to_type( tp )
-	          #^because NArray's type is not upgraded upon op with Float
-	 else  
-	   var.to_type(tp) * csf + cao
-	 end
-      end
+     unpack( simple_get(hash) )
    end
 
    def simple_put(var,hash=nil)
@@ -619,6 +615,9 @@ module NumRu
    private :__rubber_expansion
 
    def [](*a)
+     if a.length == 0
+       return self.get
+     end
      a = __rubber_expansion(a)
      first = Array.new
      last = Array.new
@@ -659,7 +658,7 @@ module NumRu
 	     a[0..at-1].each{|x| if x.is_a?(Fixnum) then k -= 1 end}
 	     shape_tmp = na_tmp.shape
 	     shape_tmp[k] = i.length
-	     na = NArray.new(na_tmp.typecode,*shape_tmp)
+	     na = na_tmp.class.new(na_tmp.typecode,*shape_tmp)
 	     index_tmp = Array.new(shape_tmp.length,true)
 	   end
 	   index_tmp[k] = n..n
@@ -730,10 +729,10 @@ module NumRu
 	   raise "length of the #{k+1}-th dim of rhs is incorrect "+
                  "(#{i.length} for #{val.shape[k]})"
 	 end
-	 index_tmp = Array.new(val.rank,true) if val.is_a?(NArray)
+	 index_tmp = Array.new(val.rank,true) if !val.is_a?(Numeric) #==>Array-like
 	 for n in 0..i.length-1
 	   a_new[at] = i[n]..i[n]
-	   if val.is_a?(NArray) then
+	   if !val.is_a?(Numeric) then
 	     index_tmp[k] = n..n
 	     self[*a_new] = val[*index_tmp]
 	   else
